#ifndef NEMO_DJBIO_H
#define NEMO_DJBIO_H

#include <nemo/stdint.h>
#include <nemo/unixtypes.h>
#include <nemo/stralloc.h>

#ifdef __cplusplus
extern "C" {
#endif

typedef struct {
  byte_t *x;
  unsigned int p;
  unsigned int n;
  int fd;
  ssize_t (*read_op)(int,void*,size_t);
  ssize_t (*write_op)(int,const void*,size_t);
} djbio;

#define DJBIO_INITREAD(op,fd,buf,len) { (byte_t*)(buf), 0, (len), (fd), (op), 0 }
#define DJBIO_INITWRITE(op,fd,buf,len) { (byte_t*)(buf), 0, (len), (fd), 0, (op) }

void	djbio_initread(register djbio *djbiop, ssize_t (*op)(int,void*,size_t), int fd, void *buf, unsigned int len);
void	djbio_initwrite(register djbio *djbiop, ssize_t (*op)(int,const void*,size_t), int fd, void *buf, unsigned int len);

int	djbio_flush(djbio *djbiop_out);
int	djbio_puteol(djbio *djbiop_out);
int	djbio_put(djbio *djbiop_out, const void *buf, unsigned int len);
int	djbio_putalign(djbio *djbiop_out, const void *buf, unsigned int len);
int	djbio_bput(djbio *djbiop_out, const void *buf, unsigned int len);
int	djbio_putflush(djbio *djbiop_out, const void *buf, unsigned int len);
int	djbio_puts(djbio *djbiop_out, const char *s);
int	djbio_putsa(djbio *djbiop_out, const stralloc *sa);
int	djbio_putsalign(djbio *djbiop_out, const char *s);
int	djbio_bputs(djbio *djbiop_out, const char *s);
int	djbio_bputsa(djbio *djbiop_out, const stralloc *sa);
int	djbio_putsflush(djbio *djbiop_out, const char *s);
int	djbio_putsaflush(djbio *djbiop_out, const stralloc *sa);

int	djbio_get(djbio *djbiop_in, void *buf, unsigned int len);
int	djbio_bget(djbio *djbiop_in, void *buf, unsigned int len);
int	djbio_feed(djbio *djbiop_in);

void	*djbio_peek(djbio *djbiop);
void	djbio_seek(djbio *djbiop, unsigned int len);

#define djbio_fileno(s) ((s)->fd)

#define DJBIO_INSIZE 8192
#define DJBIO_OUTSIZE 8192

#define djbio_PEEK(s) ( (s)->x + (s)->n )
#define djbio_SEEK(s,len) ( ( (s)->p -= (len) ), ( (s)->n += (len) ) )

#define djbio_BPUTC(s,c) \
  ( ((s)->n != (s)->p) \
    ? ( (s)->x[(s)->p++] = (c), 0 ) \
    : djbio_bput((s),&(c),1) \
  )

#define djbio_PUTC(s,c) \
  ( ((s)->n != (s)->p) \
    ? ( (s)->x[(s)->p++] = (c), 0 ) \
    : djbio_put((s),&(c),1) \
  )

#define djbio_GETC(s,c) \
  ( ((s)->p > 0) \
    ? ( *(c) = (s)->x[(s)->n], djbio_SEEK((s),1), 1 ) \
    : djbio_get((s),(c),1) \
  )

int	djbio_copy(djbio *djbiop_out, djbio *djbiop_in);

#ifdef __cplusplus
}
#endif

#endif
