#include <nemo/byte.h>
#include <nemo/error.h>

#include "djbio.h"

static int allwrite(register ssize_t (*op)(int fd, const void *buf, size_t nbytes), register int fd, register const void *in, register unsigned int len)
{
  register ssize_t w;
  register const byte_t *buf;

  buf = in;
  while (len) {
    w = op(fd, buf, len);
    if (w < 0) {
      if (errno == error_intr) continue;
      return -1; /* note that some data may have been written */
    }
    if (!w) continue;  /* { ; } */ /* luser's fault */
    buf += w;
    len -= (unsigned int)w;
  }
  return 0;
}

int djbio_flush(register djbio *iop)
{
  register unsigned int p;

  p = iop->p;
  if (!p) return 0;
  iop->p = 0;
  return allwrite(iop->write_op, iop->fd, iop->x, p);
}

int djbio_put(register djbio *iop, const void *in, register unsigned int len)
{
  register unsigned int n;
  register const byte_t *buf;

  buf = in;
  n = iop->n;
  if (len > n - iop->p) {
    if (djbio_flush(iop) < 0) return -1;
    /* now iop->p == 0 */
    if (n < DJBIO_OUTSIZE) {
      n = DJBIO_OUTSIZE;
    }
    while (len > iop->n) {
      if (n > len) {
        n = len;
      }
      if (allwrite(iop->write_op, iop->fd, buf, n) < 0) return -1;
      buf += n;
      len -= n;
    }
  }
  /* now len <= iop->n - iop->p */
  byte_copy(iop->x + iop->p, len, buf);
  iop->p += len;
  return 0;
}

int djbio_putflush(register djbio *iop, register const void *buf, register unsigned int len)
{
  if (djbio_flush(iop) < 0) return -1;
  return allwrite(iop->write_op, iop->fd, buf, len);
}
