#include <nemo/stdint.h>
#include <nemo/unixtypes.h>

#include <nemo/select.h>

#include "iopause.h"

void iopause(register iopause_fd *x, unsigned int len, const struct taia *deadline, const struct taia *stamp)
{
  register int millisecs;
  register unsigned int i;
  struct taia t;
  double d;

  if (taia_less(deadline, stamp)) {
    millisecs = 0;
  }
  else {
    t = *stamp;
    taia_sub(&t, deadline, &t);
    d = taia_approx(&t);
    if (d > 1000.0) {
      d = 1000.0;
    }
    /* millisecs = d * 1000.0 + 20.0; */
    millisecs = (int)d;
    millisecs *= 1000;
    millisecs += 20;
  }

  for (i = 0; i < len; ++i) {
    x[i].revents = 0;
  }

#ifdef IOPAUSE_POLL

  poll(x, len, millisecs);
  /* XXX: some kernels apparently need x[0] even if len is 0 */
  /* XXX: how to handle EAGAIN? are kernels really this dumb? */
  /* XXX: how to handle EINVAL? when exactly can this happen? */

#else
{

  struct timeval tv;
  fd_set rfds;
  fd_set wfds;
  int nfds;
  int fd;

  FD_ZERO(&rfds);
  FD_ZERO(&wfds);

  nfds = 1;
  for (i = 0; i < len; ++i) {
    fd = x[i].fd;
    if (fd < 0) continue;
    if (fd >= (int)(8 * sizeof(fd_set))) continue; /*XXX*/

    if (fd >= nfds) {
      nfds = fd + 1;
    }
    if (x[i].events & IOPAUSE_READ) FD_SET((unsigned int)fd, &rfds);
    if (x[i].events & IOPAUSE_WRITE) FD_SET((unsigned int)fd, &wfds);
  }

  tv.tv_sec = millisecs / 1000;
  tv.tv_usec = 1000 * (millisecs % 1000);

  if (select(nfds, &rfds, &wfds, (fd_set *) 0, &tv) <= 0) {
    return;
    /* XXX: for EBADF,  could seek out and destroy the bad descriptor */
  }

  for (i = 0; i < len; ++i) {
    fd = x[i].fd;
    if (fd < 0) continue;
    if (fd >= (int)(8 * sizeof(fd_set))) continue; /*XXX*/

    if (x[i].events & IOPAUSE_READ) {
      if (FD_ISSET((unsigned int)fd, &rfds)) {
        x[i].revents |= IOPAUSE_READ;
      }
    }
    if (x[i].events & IOPAUSE_WRITE) {
      if (FD_ISSET((unsigned int)fd, &wfds)) {
        x[i].revents |= IOPAUSE_WRITE;
      }
    }
  }

}
#endif

}
