#include <nemo/fmt.h>
#include <nemo/stralloc.h>
#include <nemo/error.h>
#include <nemo/stdint.h>
#include <nemo/unixtypes.h>
#include <nemo/unix.h>
#include <nemo/ndelay.h>

#include "socket.h"
#include "iopause.h"
#include "timeoutio.h"
#include "djbio.h"
#include "remoteinfo.h"

static unsigned int timeout;

static ssize_t mywrite(int fd, const void *buf, size_t len)
{
  return timeoutio_write(timeout, fd, buf, len);
}

static ssize_t myread(int fd, void *buf, size_t len)
{
  return timeoutio_read(timeout, fd, buf, len);
}

static int doit(stralloc *out, int s, const ip4_address *ipremote, uint16_t portremote, const ip4_address *iplocal, uint16_t portlocal)
{
  djbio b;
  char bspace[128];
  char strnum[FMT_ULONG];
  unsigned int numcolons;
  char ch;

  if (timeoutio4_connect(s, ipremote, iplocal, 113, timeout) < 0) return -1;
  ndelay_off(s);

  djbio_initwrite(&b, mywrite, s, bspace, sizeof(bspace));
  djbio_put(&b, strnum, fmt_ulong(strnum, portremote));
  djbio_put(&b, " , ", 3);
  djbio_put(&b, strnum, fmt_ulong(strnum, portlocal));
  djbio_put(&b, "\r\n", 2);
  if (djbio_flush(&b) < 0) return -1;

  djbio_initread(&b, myread, s, bspace, sizeof(bspace));
  numcolons = 0;
  for (;;) {
    if (djbio_get(&b, &ch, 1) != 1) return -1;
    if ((ch == ' ') || (ch == '\t') || (ch == '\r')) continue;
    if (ch == '\n') return 0;
    if (numcolons < 3) {
      if (ch == ':') {
        ++numcolons;
      }
    }
    else {
      if (!stralloc_append(out, &ch)) return -1;
      if (out->len > 256) return 0;
    }
  }
}

int remoteinfo4(stralloc *out, const ip4_address *ipremote, uint16_t portremote, const ip4_address *iplocal, uint16_t portlocal, unsigned int t)
{
  int s;
  int r;

  if (!stralloc_erase(out)) return -1;
  timeout = t;
  s = socket4_tcp();
  if (s < 0) return -1;
  r = doit(out, s, ipremote, portremote, iplocal, portlocal);
  close(s);
  return r;
}
