#include <nemo/stdint.h>
#include <nemo/unixtypes.h>

#include <nemo/ndelay.h>
#include <nemo/select.h>
#include <nemo/error.h>

#include "socket.h"
#include "timeoutio.h"

int timeoutio4_connect(int s, const ip4_address *ip, const ip4_address *outip, uint16_t port, unsigned int timeout)
{
  fd_set wfds;
  struct timeval tv;

  /* bind() an outgoing ipaddr */
  if (socket4_bind(s, outip, 0) < 0) return -1;

  if (ndelay_on(s) < 0) return -1;

  /* XXX: could bind s */
  if (socket4_connect(s, ip, port) == 0) {
    ndelay_off(s);
    return 0;
  }

  if ((errno != error_inprogress) && (errno != error_wouldblock)) return -1;

  FD_ZERO(&wfds);
  FD_SET((unsigned int)s, &wfds);
  tv.tv_sec = (time_t)timeout;
  tv.tv_usec = 0;

  if (select(s + 1, (fd_set*)0, &wfds, (fd_set*)0, &tv) < 0) return -1;
  if (FD_ISSET((unsigned int)s, &wfds)) {
    if (!socket_connected(s)) return -1;
    ndelay_off(s);
    return 0;
  }

  errno = error_timeout; /* note that connect attempt is continuing */
  return -1;
}
